@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -47,9 +46,11 @@
 
 #include <iostream>
 #include <algorithm>
+#include <functional>
 #include <iterator>
 #include <string>
 #include <sstream>
+#include <fstream>
 #include <vector>
 #include <iterator>
 #include <sys/types.h>
@@ -57,8 +58,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include "vigra/imageinfo.hxx"
 #include "codecmanager.hxx"
+#include "vigra/multi_impex.hxx"
 
 #if defined(_WIN32)
 #  include "vigra/windows.h"
@@ -80,11 +83,58 @@ struct NumberCompare
     }
 };
 
+bool splitString(
+    const std::string &s, char separator, std::string &a, std::string &b,
+    bool reverse = false)
+{
+    unsigned int splitPos = (reverse ? s.rfind(separator) : s.find(separator));
+    if(splitPos >= s.size())
+        return false;
+    a = std::string(s.begin(), s.begin() + splitPos);
+    b = std::string(s.begin() + splitPos + 1, s.end());
+    return true;
+}
+
+std::string trimString(const std::string &s)
+{
+    unsigned int begin = 0;
+    while(begin < s.size() && ((s[begin] == ' ') || (s[begin] == '\t')))
+        ++begin;
+    unsigned int end = s.size();
+    while(end > 0 && ((s[end-1] == ' ') || (s[end-1] == '\t')))
+        --end;
+    return std::string(s.begin() + begin, s.begin() + end);
+}
+
 } // namespace detail
 
 
 // find filenames matching the pattern "<path>/base[0-9]+ext"
 #ifdef _WIN32
+void splitPathFromFilename(const std::string &pathAndName,
+                           std::string &path, std::string &name)
+{
+	// on Windows, both '/' and '\' are valid path separators
+	// note: std::basic_string.rfind() may return 'unsigned int', so explicitely cast to 'int'
+	int split = std::max(static_cast<int>(pathAndName.rfind('/')), static_cast<int>(pathAndName.rfind('\\')));
+	if(split == static_cast<int>(std::string::npos))
+    {
+        path = ".";
+        name = pathAndName;
+    }
+    else
+    {
+        for(int i=0; i<split; ++i)
+        {
+            if(pathAndName[i] == '/')
+                path += '\\';
+            else
+                path += pathAndName[i];
+        }
+        name.append(pathAndName, split+1, pathAndName.size() - split - 1);
+    }
+}
+
 VIGRA_EXPORT void findImageSequence(const std::string &name_base,
                        const std::string &name_ext,
                        std::vector<std::string> & numbers)
@@ -95,27 +145,8 @@ VIGRA_EXPORT void findImageSequence(const std::string &name_base,
     TCHAR           szDir[MAX_PATH+1];
     WIN32_FIND_DATA FileData;
 
-    std::string base, path;
-
-	// on Windows, both '/' and '\' are valid path separators
-	// note: std::basic_string.rfind() may return 'unsigned int', so explicitely cast to 'int'
-	int split = std::max(static_cast<int>(name_base.rfind('/')), static_cast<int>(name_base.rfind('\\')));
-	if(split == static_cast<int>(std::string::npos))
-    {
-        path = ".";
-        base = name_base;
-    }
-    else
-    {
-        for(int i=0; i<split; ++i)
-        {
-            if(name_base[i] == '/')
-                path += '\\';
-            else
-                path += name_base[i];
-        }
-        base.append(name_base, split+1, name_base.size() - split - 1);
-    }
+    std::string path, base;
+	splitPathFromFilename(name_base, path, base);
 
     std::vector<std::string> result;
     char numbuf[21], extbuf[1024];
@@ -141,7 +172,13 @@ VIGRA_EXPORT void findImageSequence(const std::string &name_base,
             if(sscanf(FileData.cFileName, pattern.c_str(), numbuf, extbuf) == 2)
             {
                 if(strcmp(name_ext.c_str(), extbuf) == 0)
-                    result.push_back(std::string(numbuf));
+                {
+                    std::string num(numbuf);
+                    std::string name = name_base + num + name_ext;
+                    // skip matching files names that are not images
+                    if(isImage(name.c_str()))
+                        result.push_back(num);
+                }
             }
             if (!FindNextFile(hList, &FileData))
             {
@@ -161,23 +198,29 @@ VIGRA_EXPORT void findImageSequence(const std::string &name_base,
 
 #else // _WIN32
 
+void splitPathFromFilename(const std::string &pathAndName,
+                           std::string &path, std::string &name)
+{
+    int split = pathAndName.rfind('/');
+    if(split == -1)
+    {
+        path = ".";
+        name = pathAndName;
+    }
+    else
+    {
+        path.append(pathAndName, 0, split);
+        name.append(pathAndName, split+1, pathAndName.size() - split - 1);
+    }
+}
+
 void findImageSequence(const std::string &name_base,
                        const std::string &name_ext,
                        std::vector<std::string> & numbers)
 {
     // find out how many images we have
-    std::string base, path;
-    int split = name_base.rfind('/');
-    if(split == -1)
-    {
-        path = ".";
-        base = name_base;
-    }
-    else
-    {
-        path.append(name_base, 0, split);
-        base.append(name_base, split+1, name_base.size() - split - 1);
-    }
+    std::string path, base;
+	splitPathFromFilename(name_base, path, base);
 
     DIR * dir = opendir(path.c_str());
     if(!dir)
@@ -197,7 +240,13 @@ void findImageSequence(const std::string &name_base,
         if(sscanf(dp->d_name, pattern.c_str(), numbuf, extbuf) == 2)
         {
             if(strcmp(name_ext.c_str(), extbuf) == 0)
-                result.push_back(std::string(numbuf));
+            {
+                std::string num(numbuf);
+                std::string name = name_base + num + name_ext;
+                // skip matching files names that are not images
+                if(isImage(name.c_str()))
+                    result.push_back(num);
+            }
         }
     }
 
@@ -274,7 +323,8 @@ bool isImage(char const * filename)
 
 ImageExportInfo::ImageExportInfo( const char * filename )
     : m_filename(filename),
-      m_x_res(0), m_y_res(0)
+      m_x_res(0), m_y_res(0),
+      fromMin_(0.0), fromMax_(0.0), toMin_(0.0), toMax_(0.0)
 {}
 
 ImageExportInfo::~ImageExportInfo()
@@ -287,12 +337,53 @@ ImageExportInfo & ImageExportInfo::setFileType( const char * filetype )
     return *this;
 }
 
+ImageExportInfo & ImageExportInfo::setForcedRangeMapping(double fromMin, double fromMax,
+                                                     double toMin, double toMax)
+{
+    fromMin_ = fromMin;
+    fromMax_ = fromMax;
+    toMin_ = toMin;
+    toMax_ = toMax;
+    return *this;
+}
+
+bool ImageExportInfo::hasForcedRangeMapping() const
+{
+    return fromMax_ > fromMin_;
+}
+
+double ImageExportInfo::getFromMin() const
+{
+    return fromMin_;
+}
+
+double ImageExportInfo::getFromMax() const
+{
+    return fromMax_;
+}
+
+double ImageExportInfo::getToMin() const
+{
+    return toMin_;
+}
+
+double ImageExportInfo::getToMax() const
+{
+    return toMax_;
+}
+
 ImageExportInfo & ImageExportInfo::setCompression( const char * comp )
 {
     m_comp = comp;
     return *this;
 }
 
+ImageExportInfo & ImageExportInfo::setFileName(const char * name)
+{
+    m_filename = name;
+    return *this;
+}
+
 const char * ImageExportInfo::getFileName() const
 {
     return m_filename.c_str();
@@ -550,4 +641,156 @@ std::auto_ptr<Decoder> decoder( const ImageImportInfo & info )
     return getDecoder( std::string( info.getFileName() ), filetype );
 }
 
+VolumeImportInfo::VolumeImportInfo(const std::string &filename)
+: shape_(0, 0, 0),
+  resolution_(1.f, 1.f, 1.f),
+  numBands_(0)
+{
+    // first try image sequence loading
+    std::string::const_reverse_iterator
+        numBeginIt(filename.rbegin()), numEndIt(numBeginIt);
+
+    do
+    {
+        numEndIt = std::find_if(numBeginIt, filename.rend(), &isdigit);
+        numBeginIt = std::find_if(numEndIt, filename.rend(), not1(std::ptr_fun(&isdigit)));
+
+        if(numEndIt != filename.rend())
+        {
+            std::string
+                baseName(filename.begin(),
+                         filename.begin() + (filename.rend()-numBeginIt)),
+                extension(filename.begin() + (filename.rend()-numEndIt),
+                          filename.end());
+
+            std::vector<std::string> numbers;
+
+            findImageSequence(baseName, extension, numbers);
+            if(numbers.size() > 0)
+            {
+                getVolumeInfoFromFirstSlice(baseName + numbers[0] + extension);
+                splitPathFromFilename(baseName, path_, name_);
+                baseName_ = baseName;
+                extension_ = extension;
+                shape_[2] = numbers.size();
+                std::swap(numbers, numbers_);
+
+                break;
+            }
+        }
+    }
+    while(numEndIt != filename.rend());
+
+    // no numbered images found, try .info file loading
+    if(!numbers_.size())
+    {
+        std::ifstream stream(filename.c_str());
+
+        while(stream.good())
+        {
+            char rawline[1024];
+            stream.getline(rawline, 1024);
+
+            // split off comments starting with '#':
+            std::string line, comment;
+            if(!detail::splitString(rawline, '#', line, comment))
+                line = rawline;
+
+            std::string key, value;
+            if(detail::splitString(line, '=', key, value))
+            {
+                key = detail::trimString(key);
+                value = detail::trimString(value);
+
+                if(key == "width")
+                    shape_[0] = atoi(value.c_str());
+                else if(key == "height")
+                    shape_[1] = atoi(value.c_str());
+                else if(key == "depth")
+                    shape_[2] = atoi(value.c_str());
+                else if(key == "datatype")
+                {
+                    // FUTURE: store bit depth / signedness
+                    if((value == "UNSIGNED_CHAR") || (value == "UNSIGNED_BYTE"))
+                        numBands_ = 1;
+                    else
+                    {
+                        std::cerr << "Unknown datatype '" << value << "'!\n";
+                        break;
+                    }
+                }
+                else if(key == "description")
+                    description_ = value;
+                else if(key == "name")
+                    name_ = value;
+                else if(key == "filename")
+                    rawFilename_ = value;
+                else
+                {
+                    std::cerr << "WARNING: Unknown key '" << key
+                              << "' (value '" << value << "') in info file!\n";
+                }
+            }
+            else
+            {
+                if(line[0]) // non-empty line?
+                    std::cerr << "WARNING: could not parse line '" << line << "'!\n";
+            }
+        }
+
+        if((shape_[0]*shape_[1]*shape_[2] > 0) && (rawFilename_.size() > 0))
+        {
+            if(!numBands_)
+                numBands_ = 1; // default to UNSIGNED_CHAR datatype
+
+            baseName_ = filename;
+            if(name_.size() > 0)
+            {
+                std::string nameDummy;
+                splitPathFromFilename(baseName_, path_, nameDummy);
+            }
+            else
+            {
+                splitPathFromFilename(baseName_, path_, name_);
+            }
+            return;
+        }
+
+        std::string message("VolumeImportInfo(): Unable to load volume '");
+        message += filename + "'.";
+        vigra_fail(message.c_str());
+    }
+}
+
+VolumeImportInfo::VolumeImportInfo(const std::string &baseName, const std::string &extension)
+: shape_(0, 0, 0),
+  resolution_(1.f, 1.f, 1.f),
+  numBands_(0)
+{
+    std::vector<std::string> numbers;
+    findImageSequence(baseName, extension, numbers);
+
+    std::string message("VolumeImportInfo(): No files matching '");
+    message += baseName + "[0-9]+" + extension + "' found.";
+    vigra_precondition(numbers.size() > 0, message.c_str());
+
+    getVolumeInfoFromFirstSlice(baseName + numbers[0] + extension);
+
+    splitPathFromFilename(baseName, path_, name_);
+    baseName_ = baseName;
+    extension_ = extension;
+    shape_[2] = numbers.size();
+    std::swap(numbers, numbers_);
+}
+
+void VolumeImportInfo::getVolumeInfoFromFirstSlice(const std::string &filename)
+{
+    ImageImportInfo info(filename.c_str());
+    shape_[0] = info.width();
+    shape_[1] = info.height();
+    resolution_[1] = -1.f; // assume images to be right-handed
+    pixelType_ = info.pixelType();
+    numBands_ = info.numBands();
+}
+
 } // namespace vigra
