@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -43,6 +42,7 @@
 #include "vigra/impex.hxx"
 #include "vigra/combineimages.hxx"
 #include "vigra/resampling_convolution.hxx"
+#include "vigra/imagecontainer.hxx"
 
 using namespace vigra;
 
@@ -138,6 +138,72 @@ struct ConvolutionTest
             acc.set(k, i);
         }
     }
+    
+    void initExplicitlyTest()
+    {
+        vigra::Kernel1D<double> k;
+        k.initExplicitly(-1,2) = 1,2,3,4;
+        
+        shouldEqual(k.left(), -1);
+        shouldEqual(k.right(), 2);
+        shouldEqual(k[-1], 1);
+        shouldEqual(k[0], 2);
+        shouldEqual(k[1], 3);
+        shouldEqual(k[2], 4);
+        
+        k.initExplicitly(-2,1) = -2;
+        shouldEqual(k.left(), -2);
+        shouldEqual(k.right(), 1);
+        shouldEqual(k[-2], -2);
+        shouldEqual(k[-1], -2);
+        shouldEqual(k[0], -2);
+        shouldEqual(k[1], -2);
+
+        try
+        {
+            k.initExplicitly(-1,1) = 1, 2;
+            failTest("no exception thrown");
+        }
+        catch(vigra::PreconditionViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nKernel1D::initExplicitly(): Wrong number of init values.");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+        try
+        {
+            k.initExplicitly(-1,1) = 1, 2, 3, 4;
+            failTest("no exception thrown");
+        }
+        catch(vigra::PreconditionViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nKernel1D::initExplicitly(): Wrong number of init values.");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+        try
+        {
+            k.initExplicitly(1,1) = 1, 2, 3, 4;
+            failTest("no exception thrown");
+        }
+        catch(vigra::PreconditionViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nKernel1D::initExplicitly(): left border must be <= 0.");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+        try
+        {
+            k.initExplicitly(-1,-1) = 1, 2, 3, 4;
+            failTest("no exception thrown");
+        }
+        catch(vigra::PreconditionViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nKernel1D::initExplicitly(): right border must be >= 0.");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+    }
 
     void simpleSharpeningTest(){
          Image dest_lenna(lenna);
@@ -700,6 +766,54 @@ struct ConvolutionTest
         }
     }
     
+    void optimalSmoothing3Test()
+    {
+        vigra::Kernel1D<double> smooth3;
+        smooth3.initOptimalSmoothing3();
+
+        Image tmp1(lenna.size());
+        Image tmp2(lenna.size());
+
+        separableConvolveX(srcImageRange(lenna), destImage(tmp1), kernel1d(smooth3));
+        separableConvolveY(srcImageRange(tmp1), destImage(tmp2), kernel1d(smooth3));
+        
+        gaussianSmoothing(srcImageRange(lenna), destImage(tmp1), 0.68);
+        
+        Image::ScanOrderIterator i1 = tmp1.begin();
+        Image::ScanOrderIterator i1end = tmp1.end();
+        Image::ScanOrderIterator i2 = tmp2.begin();
+        Image::Accessor acc = constimg.accessor();
+        
+        for(; i1 != i1end; ++i1, ++i2)
+        {
+            shouldEqualTolerance(acc(i1), acc(i2), 1e-2);
+        }
+    }
+    
+    void optimalSmoothing5Test()
+    {
+        vigra::Kernel1D<double> smooth5;
+        smooth5.initOptimalSmoothing5();
+
+        Image tmp1(lenna.size());
+        Image tmp2(lenna.size());
+
+        separableConvolveX(srcImageRange(lenna), destImage(tmp1), kernel1d(smooth5));
+        separableConvolveY(srcImageRange(tmp1), destImage(tmp2), kernel1d(smooth5));
+        
+        gaussianSmoothing(srcImageRange(lenna), destImage(tmp1), 0.867);
+        
+        Image::ScanOrderIterator i1 = tmp1.begin();
+        Image::ScanOrderIterator i1end = tmp1.end();
+        Image::ScanOrderIterator i2 = tmp2.begin();
+        Image::Accessor acc = constimg.accessor();
+        
+        for(; i1 != i1end; ++i1, ++i2)
+        {
+            shouldEqualTolerance(acc(i1), acc(i2), 1e-2);
+        }
+    }
+    
     void separableGradientTest()
     {
         Image sepgrad(lenna.size());
@@ -732,7 +846,7 @@ struct ConvolutionTest
         {
             double grad = VIGRA_CSTD::sqrt(acc(i1)*acc(i1)+acc(i2)*acc(i2));
 
-            shouldEqualTolerance(grad, acc(i), 1e-7);
+            shouldEqualTolerance(grad-acc(i), 0.0, 1e-12);
         }
     }
     
@@ -759,8 +873,140 @@ struct ConvolutionTest
         for(; i1 != i1end; ++i1, ++i2, ++ig, ++i)
         {
             double grad = VIGRA_CSTD::sqrt(acc(i1)*acc(i1)+acc(i2)*acc(i2));
-            shouldEqualTolerance(grad, acc(i), 1e-7);
-            shouldEqualTolerance(acc(ig), acc(i), 1e-7);
+            shouldEqualTolerance(grad-acc(i), 0.0, 1e-12);
+            shouldEqualTolerance(acc(ig)-acc(i), 0.0, 1e-12);
+        }
+    }
+    
+    void optimalGradient3Test()
+    {
+        Image tmp(lenna.size());
+        Image tmpx(lenna.size());
+        Image tmpy(lenna.size());
+
+        vigra::Kernel1D<double> diff;
+        diff.initSymmetricDifference();
+        vigra::Kernel1D<double> smooth3;
+        smooth3.initOptimalFirstDerivativeSmoothing3();
+
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(diff));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpx), kernel1d(smooth3));
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(smooth3));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpy), kernel1d(diff));
+
+        double sum = 0.0, mi = 0.0, ma = 0.0;
+
+        Image::ScanOrderIterator i1 = tmpx.begin();
+        Image::ScanOrderIterator i1end = tmpx.end();
+        Image::ScanOrderIterator i2 = tmpy.begin();
+        Image::Accessor acc = constimg.accessor();
+
+        for(; i1 != i1end; ++i1, ++i2)
+        {
+            double grad = VIGRA_CSTD::sqrt(acc(i1)*acc(i1)+acc(i2)*acc(i2));
+            sum += grad;
+            mi = std::min(mi, grad);
+            ma = std::max(ma, grad);
+        }
+        should(std::fabs(sum- 130000.0) < 1000.0);
+        shouldEqual(mi, 0.0);
+        should(std::fabs(ma - 68.0) < 1.0);
+    }
+    
+    void optimalLaplacian3Test()
+    {
+        Image tmp(lenna.size());
+        Image tmpx(lenna.size());
+        Image tmpy(lenna.size());
+
+        vigra::Kernel1D<double> diff;
+        diff.initSecondDifference3();
+        vigra::Kernel1D<double> smooth3;
+        smooth3.initOptimalSecondDerivativeSmoothing3();
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(diff));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpx), kernel1d(smooth3));
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(smooth3));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpy), kernel1d(diff));
+
+        double sum = 0.0, mi = 0.0, ma = 0.0;
+
+        Image::ScanOrderIterator i1 = tmpx.begin();
+        Image::ScanOrderIterator i1end = tmpx.end();
+        Image::ScanOrderIterator i2 = tmpy.begin();
+        Image::Accessor acc = constimg.accessor();
+
+        for(; i1 != i1end; ++i1, ++i2)
+        {
+            double lap = acc(i1) + acc(i2);
+            sum += lap;
+            mi = std::min(mi, lap);
+            ma = std::max(ma, lap);
+        }
+        should(std::fabs(sum) < 90.0);
+        should(std::fabs(mi + 120.0) < 1.0);
+        should(std::fabs(ma - 117.0) < 1.0);
+    }
+        
+    void optimalGradient5Test()
+    {
+        Image tmp(lenna.size());
+        Image tmpx(lenna.size());
+        Image tmpy(lenna.size());
+        Image mag(lenna.size());
+
+        vigra::Kernel1D<double> diff;
+        diff.initOptimalFirstDerivative5();
+        vigra::Kernel1D<double> smooth5;
+        smooth5.initOptimalFirstDerivativeSmoothing5();
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(diff));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpx), kernel1d(smooth5));
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(smooth5));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpy), kernel1d(diff));
+
+        gaussianGradientMagnitude(srcImageRange(lenna), destImage(mag), 0.906);
+
+        for(int y=1; y<lenna.height()-1; ++y)
+        {
+            for(int x=1; x<lenna.width()-1; ++x)
+            {
+                double grad = vigra::hypot(tmpx(x,y), tmpy(x,y));
+                should(std::fabs(grad-mag(x,y)) < 2.0);
+            }
+        }
+    }
+    
+    void optimalLaplacian5Test()
+    {
+        Image tmp(lenna.size());
+        Image tmpx(lenna.size());
+        Image tmpy(lenna.size());
+        Image lap(lenna.size());
+
+        vigra::Kernel1D<double> diff;
+        diff.initOptimalSecondDerivative5();
+        vigra::Kernel1D<double> smooth5;
+        smooth5.initOptimalSecondDerivativeSmoothing5();
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(diff));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpx), kernel1d(smooth5));
+        
+        separableConvolveX(srcImageRange(lenna), destImage(tmp), kernel1d(smooth5));
+        separableConvolveY(srcImageRange(tmp), destImage(tmpy), kernel1d(diff));
+
+        laplacianOfGaussian(srcImageRange(lenna), destImage(lap), 0.817);
+
+        for(int y=2; y<lenna.height()-2; ++y)
+        {
+            for(int x=2; x<lenna.width()-2; ++x)
+            {
+                double l = tmpx(x,y) + tmpy(x,y);
+                should(std::fabs(l-lap(x,y)) < 4.0);
+            }
         }
     }
     
@@ -776,6 +1022,7 @@ struct ConvolutionTest
         RGBImage tmpx(lenna.size());
         RGBImage tmpy(lenna.size());
         Image mag(lenna.size());
+        Image resmag(lenna.size());
 
         gaussianGradient(srcImageRange(input), destImage(tmpx), destImage(tmpy), 1.0);
         gaussianGradientMagnitude(srcImageRange(input), destImage(mag), 1.0);
@@ -791,10 +1038,11 @@ struct ConvolutionTest
         for(; i1 != i1end; ++i1, ++i2, ++ig, ++i)
         {
             double grad = VIGRA_CSTD::sqrt(squaredNorm(rgb(i1))+squaredNorm(rgb(i2)));
-            shouldEqualTolerance(grad, acc(i), 1e-6);
-            shouldEqualTolerance(acc(ig), acc(i), 1e-6);
+            shouldEqualTolerance(grad-acc(i), 0.0, 1e-12);
+            shouldEqualTolerance(acc(ig)-acc(i), 0.0, 1e-12);
         }
-    }
+
+   }
     
     void hessianTest()
     {
@@ -824,9 +1072,9 @@ struct ConvolutionTest
         
         for(; i1 != i1end; ++i1, ++i2, ++i3, ++r1, ++r2, ++r3)
         {
-                shouldEqualTolerance(acc(i1), acc(r1), 1e-7);
-                shouldEqualTolerance(acc(i2), acc(r2), 1e-7);
-                shouldEqualTolerance(acc(i3), acc(r3), 1e-7);
+            shouldEqualTolerance(acc(i1)-acc(r1), 0.0, 1e-12);
+            shouldEqualTolerance(acc(i2)-acc(r2), 0.0, 1e-12);
+            shouldEqualTolerance(acc(i3)-acc(r3), 0.0, 1e-12);
         }
     }
     
@@ -838,9 +1086,14 @@ struct ConvolutionTest
         Image refxx(lenna.size());
         Image refxy(lenna.size());
         Image refyy(lenna.size());
+        
+        typedef BasicImage<TinyVector<double, 3> > VectorImage;
+        VectorImage resst(lenna.size());
 
         structureTensor(srcImageRange(lenna), 
             destImage(resxx), destImage(resxy), destImage(resyy), 1.0, 2.0);
+
+        structureTensor(srcImageRange(lenna), destImage(resst), 1.0, 2.0);
             
         importImage(vigra::ImageImportInfo("lennastxx.xv"), destImage(refxx));
         importImage(vigra::ImageImportInfo("lennastyy.xv"), destImage(refyy));
@@ -848,18 +1101,48 @@ struct ConvolutionTest
         
         Image::ScanOrderIterator i1 = resxx.begin();
         Image::ScanOrderIterator i1end = resxx.end();
-        Image::ScanOrderIterator i2 = resyy.begin();
-        Image::ScanOrderIterator i3 = resxy.begin();
+        Image::ScanOrderIterator i2 = resxy.begin();
+        Image::ScanOrderIterator i3 = resyy.begin();
+        VectorImage::ScanOrderIterator i4 = resst.begin();
         Image::ScanOrderIterator r1 = refxx.begin();
-        Image::ScanOrderIterator r2 = refyy.begin();
-        Image::ScanOrderIterator r3 = refxy.begin();
+        Image::ScanOrderIterator r2 = refxy.begin();
+        Image::ScanOrderIterator r3 = refyy.begin();
         Image::Accessor acc = constimg.accessor();
+        VectorImage::Accessor vacc = resst.accessor();
 
-        for(; i1 != i1end; ++i1, ++i2, ++i3, ++r1, ++r2, ++r3)
+        for(; i1 != i1end; ++i1, ++i2, ++i3, ++i4, ++r1, ++r2, ++r3)
         {
-                shouldEqualTolerance(acc(i1), acc(r1), 1e-7);
-                shouldEqualTolerance(acc(i2), acc(r2), 1e-7);
-                shouldEqualTolerance(acc(i3), acc(r3), 1e-7);
+            shouldEqualTolerance(acc(i1), acc(r1), 1e-7);
+            shouldEqualTolerance(acc(i2), acc(r2), 1e-7);
+            shouldEqualTolerance(acc(i3), acc(r3), 1e-7);
+            shouldEqualTolerance(vacc(i4)[0], acc(r1), 1e-7);
+            shouldEqualTolerance(vacc(i4)[1], acc(r2), 1e-7);
+            shouldEqualTolerance(vacc(i4)[2], acc(r3), 1e-7);
+        }
+    }
+    
+    void structureTensorRGBTest()
+    {
+        RGBImage input(lenna.size());
+        importImage(vigra::ImageImportInfo("lenna128rgb.xv"), destImage(input));
+
+        typedef BasicImage<TinyVector<double, 3> > VectorImage;
+        VectorImage resst(lenna.size()), refst(lenna.size());
+
+        structureTensor(srcImageRange(lenna), destImage(resst), 1.0, 2.0);
+        
+        importImage(vigra::ImageImportInfo("lennargbst.xv"), destImage(refst));
+        
+        VectorImage::ScanOrderIterator i1 = resst.begin();
+        VectorImage::ScanOrderIterator i1end = resst.end();
+        VectorImage::ScanOrderIterator r1 = refst.begin();
+        VectorImage::Accessor vacc = resst.accessor();
+
+        for(; i1 != i1end; ++i1, ++r1)
+        {
+            shouldEqualTolerance(vacc(i1)[0], vacc(r1)[0], 1e-7);
+            shouldEqualTolerance(vacc(i1)[1], vacc(r1)[1], 1e-7);
+            shouldEqualTolerance(vacc(i1)[2], vacc(r1)[2], 1e-7);
         }
     }
     
@@ -1711,12 +1994,89 @@ struct ResamplingConvolutionTest
     }
 };
 
+struct ImagePyramidTest
+{
+    typedef vigra::DImage Image;
+    Image img;
+    int w, h;
+
+    ImagePyramidTest()
+    {
+        ImageImportInfo ginfo("lenna128.xv");
+        w = ginfo.width();
+        h = ginfo.height();
+        img.resize(w, h);
+        importImage(ginfo, destImage(img));
+    }
+
+    void testPyramidConstruction()
+    {
+        vigra::ImagePyramid<Image> pyramid(-2, 2, img);
+        
+        shouldEqual(pyramid[-2].size(), Size2D(509, 477));
+        shouldEqual(pyramid[-1].size(), Size2D(255, 239));
+        shouldEqual(pyramid[0].size(), Size2D(128, 120));
+        shouldEqual(pyramid[1].size(), Size2D(64, 60));
+        shouldEqual(pyramid[2].size(), Size2D(32, 30));
+    }
+
+    void testBurtReduceExpand()
+    {
+        vigra::ImagePyramid<Image> pyramid(-2, 3, img), laplacian(-2,3, img);
+        
+        pyramidExpandBurtFilter(pyramid, 0, -2);
+        pyramidReduceBurtFilter(pyramid, 0,  3);
+        
+        pyramidReduceBurtLaplacian(laplacian, 0, 3);
+
+        char buf[100];
+
+        for(int i=-2; i<=2; ++i)
+        {
+            if(i==0)
+                continue;
+
+            std::sprintf(buf, "lenna_level%d.xv", i);
+            ImageImportInfo info(buf);
+            shouldEqual(info.size(), pyramid[i].size());
+            
+            Image ref(info.size());
+            importImage(info, destImage(ref));
+            shouldEqualSequenceTolerance(ref.begin(), ref.end(), pyramid[i].begin(), 1e-12);
+        }
+        
+        for(int i=0; i<=2; ++i)
+        {
+            std::sprintf(buf, "lenna_levellap%d.xv", i);
+            ImageImportInfo info(buf);
+            shouldEqual(info.size(), laplacian[i].size());
+            
+            Image ref(info.size());
+            importImage(info, destImage(ref));
+            for(int k=0; k<info.width()*info.height(); ++k)
+                shouldEqualTolerance(ref.data()[k]-laplacian[i].data()[k], 0.0, 1e-12);
+        }
+        
+        shouldEqualSequenceTolerance(pyramid[3].begin(), pyramid[3].end(), laplacian[3].begin(), 1e-14);
+        
+        pyramidExpandBurtLaplacian(laplacian, 3, -2);
+
+        for(int i=3; i>=-2; --i)
+        {
+            shouldEqualSequenceTolerance(pyramid[i].begin(), pyramid[i].end(), laplacian[i].begin(), 1e-14);
+        }
+    }
+        
+};
+
 struct ConvolutionTestSuite
 : public vigra::test_suite
 {
     ConvolutionTestSuite()
     : vigra::test_suite("ConvolutionTestSuite")
     {
+        add( testCase( &ConvolutionTest::initExplicitlyTest));
+
         add( testCase( &ConvolutionTest::simpleSharpeningTest)); 
         add( testCase( &ConvolutionTest::gaussianSharpeningTest)); 
         add( testCase( &ConvolutionTest::stdConvolutionTestOnConstImage));
@@ -1736,11 +2096,18 @@ struct ConvolutionTestSuite
         add( testCase( &ConvolutionTest::separableSmoothClipTest));
         add( testCase( &ConvolutionTest::separableSmoothWrapTest));
         add( testCase( &ConvolutionTest::gaussianSmoothingTest));
+        add( testCase( &ConvolutionTest::optimalSmoothing3Test));
+        add( testCase( &ConvolutionTest::optimalSmoothing5Test));
+        add( testCase( &ConvolutionTest::optimalGradient3Test));
+        add( testCase( &ConvolutionTest::optimalGradient5Test));
+        add( testCase( &ConvolutionTest::optimalLaplacian3Test));
+        add( testCase( &ConvolutionTest::optimalLaplacian5Test));
         add( testCase( &ConvolutionTest::separableGradientTest));
         add( testCase( &ConvolutionTest::gradientTest));
         add( testCase( &ConvolutionTest::gradientRGBTest));
         add( testCase( &ConvolutionTest::hessianTest));
         add( testCase( &ConvolutionTest::structureTensorTest));
+        add( testCase( &ConvolutionTest::structureTensorRGBTest));
         add( testCase( &ConvolutionTest::stdConvolutionTest));
         add( testCase( &ConvolutionTest::stdVersusSeparableConvolutionTest));
         add( testCase( &ConvolutionTest::recursiveFilterTestWithAvoid));
@@ -1761,6 +2128,9 @@ struct ConvolutionTestSuite
         add( testCase( &ResamplingConvolutionTest::testKernelsGauss));
         add( testCase( &ResamplingConvolutionTest::testOversamplingConstant));
         add( testCase( &ResamplingConvolutionTest::testOversamplingReal));
+
+        add( testCase( &ImagePyramidTest::testPyramidConstruction));
+        add( testCase( &ImagePyramidTest::testBurtReduceExpand));
     }
 };
 
