@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -38,6 +37,7 @@
 #include <iostream>
 #include <cmath>
 #include <cstdlib>
+#include <cstring>
 #include "vigra/stdimage.hxx"
 #include "vigra/impex.hxx"
 #include "unittest.hxx"
@@ -524,11 +524,13 @@ class PNGInt16Test
 
 class FloatImageExportImportTest
 {
-    typedef vigra::FImage Image;
+    typedef vigra::DImage Image;
+    std::string rereadType;
 
 public:
 
     FloatImageExportImportTest ()
+    : rereadType("DOUBLE")
     {
         vigra::ImageImportInfo info ("lenna.xv");
 
@@ -607,7 +609,7 @@ public:
         should (info.width () == img.width ());
         should (info.height () == img.height ());
         should (info.isGrayscale ());
-        should (info.getPixelType () == std::string ("FLOAT"));
+        should (info.getPixelType () == rereadType);
 
         Image res (info.width (), info.height ());
 
@@ -618,12 +620,30 @@ public:
         Image::Accessor acc = img.accessor ();
 
         shouldEqualSequence(i, img.end(), i1);
+    }
 
- /*       for (; i != img.end (); ++i, ++i1)
-            {
-                shouldEqual(acc (i), acc (i1));
-            }
- */
+    void testTIFFForcedRange ()
+    {
+        exportImage (srcImageRange (img),
+                     vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0, 255, 1, 2));
+
+        vigra::ImageImportInfo info ("res.tif");
+
+        should (info.width () == img.width ());
+        should (info.height () == img.height ());
+        should (info.isGrayscale ());
+        should (info.getPixelType () == rereadType);
+
+        Image res (info.width (), info.height ());
+
+        importImage (info, destImage (res));
+
+        Image::ScanOrderIterator i = img.begin ();
+        Image::ScanOrderIterator i1 = res.begin ();
+        Image::Accessor acc = img.accessor ();
+
+        for (; i != img.end (); ++i, ++i1)
+            shouldEqualTolerance(acc (i) / 255.0, acc (i1) - 1.0, 1e-12);
     }
 
     void testBMP ()
@@ -683,7 +703,7 @@ public:
         should (info.width () == img.width ());
         should (info.height () == img.height ());
         should (info.isGrayscale ());
-        should (info.getPixelType () == std::string ("FLOAT"));
+        should (info.getPixelType () == rereadType);
 
         Image res (info.width (), info.height ());
 
@@ -777,6 +797,34 @@ public:
             shouldEqual (acc (i), acc (i1));
     }
 
+    void testTIFFForcedRange ()
+    {
+        exportImage (srcImageRange (img),
+                     vigra::ImageExportInfo ("res.tif").setForcedRangeMapping(0,255,1,2));
+
+        vigra::ImageImportInfo info ("res.tif");
+
+        should (info.width () == img.width ());
+        should (info.height () == img.height ());
+        should (info.isColor ());
+        should (info.getPixelType () == std::string ("FLOAT"));
+
+        Image res (info.width (), info.height ());
+
+        importImage (info, destImage (res));
+
+        Image::ScanOrderIterator i = img.begin ();
+        Image::ScanOrderIterator i1 = res.begin ();
+        Image::Accessor acc = img.accessor ();
+
+        for (; i != img.end (); ++i, ++i1)
+        {
+            shouldEqualTolerance(acc.red(i)/255.0f, acc.red(i1)-1.0f, 1e-4);
+            shouldEqualTolerance(acc.green(i)/255.0f, acc.green(i1)-1.0f, 1e-4);
+            shouldEqualTolerance(acc.blue(i)/255.0f, acc.blue(i1)-1.0f, 1e-4);
+        }
+    }
+
     void testBMP ()
     {
         exportImage (srcImageRange (img), vigra::ImageExportInfo ("res.bmp"));
@@ -1211,6 +1259,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         add(testCase(&FloatImageExportImportTest::testGIF));
         add(testCase(&FloatImageExportImportTest::testJPEG));
         add(testCase(&FloatImageExportImportTest::testTIFF));
+        add(testCase(&FloatImageExportImportTest::testTIFFForcedRange));
         add(testCase(&FloatImageExportImportTest::testBMP));
         add(testCase(&FloatImageExportImportTest::testSUN));
         add(testCase(&FloatImageExportImportTest::testVIFF));
@@ -1228,6 +1277,7 @@ struct ImageImportExportTestSuite : public vigra::test_suite
         // rgb float images
         add(testCase(&FloatRGBImageExportImportTest::testJPEG));
         add(testCase(&FloatRGBImageExportImportTest::testTIFF));
+        add(testCase(&FloatRGBImageExportImportTest::testTIFFForcedRange));
         add(testCase(&FloatRGBImageExportImportTest::testBMP));
         add(testCase(&FloatRGBImageExportImportTest::testSUN));
         add(testCase(&FloatRGBImageExportImportTest::testVIFF));
