@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -43,7 +42,6 @@
 
 using namespace vigra;
 
-
 class MultiArrayDataTest
 {
 public:
@@ -53,8 +51,10 @@ public:
     // ascending numbers.
 
     typedef unsigned int scalar_type;
+    typedef MultiArray <2, scalar_type> array2_type;
     typedef MultiArray <3, scalar_type> array3_type;
     typedef MultiArrayView <3, scalar_type> array3_view_type;
+    typedef array2_type::difference_type difference2_type;
     typedef array3_type::difference_type difference3_type;
     
     difference3_type shape3;
@@ -68,6 +68,17 @@ public:
             array3.data () [i] = i;
     }
     
+    void testEquality ()
+    {
+        typedef difference3_type Shape;
+        should(array3 == array3);
+        should(array3 != array3.subarray(Shape(1,1,1), Shape(2,2,2)));
+        should(array3.subarray(Shape(0,0,0), Shape(10,1,1)) != array3.subarray(Shape(0,1,0), Shape(10,2,1)));
+        for(unsigned int k=0; k<10; ++k)
+            array3(k,0,0) += 10;
+        should(array3.subarray(Shape(0,0,0), Shape(10,1,1)) == array3.subarray(Shape(0,1,0), Shape(10,2,1)));
+    }
+    
     // bindInner tests
     void test_bindInner ()
     {
@@ -133,14 +144,24 @@ public:
     // subarray tests
     void test_subarray ()
     {
-        difference3_type offset (1,1,1);
-        difference3_type size (5,5,5);
+        typedef difference3_type Shape;
+        
+        Shape offset (1,1,1);
+        Shape size (5,5,5);
         MultiArrayView <3, scalar_type, UnstridedArrayTag>
             array = array3.subarray (offset, size);
-        shouldEqual (array [difference3_type (0,0,0)], 111);
-        shouldEqual (array [difference3_type (5,2,1)], 236);
+        shouldEqual (array [Shape (0,0,0)], 111);
+        shouldEqual (array [Shape (5,2,1)], 236);
         shouldEqual (array (0,0,0), 111);
         shouldEqual (array (5,2,1), 236);
+        
+        // test swap
+        array3.subarray(Shape(0,0,0), Shape(10,10,1)).swapData( 
+             array3.subarray(Shape(0,0,1), Shape(10,10,2)));
+        for(unsigned int k=0; k<100; ++k)
+            shouldEqual(array3[k], k+100);
+        for(unsigned int k=100; k<200; ++k)
+            shouldEqual(array3[k], k-100);
     }
         
     // stridearray tests
@@ -159,6 +180,139 @@ public:
         shouldEqual (array [traverser], 222);
         shouldEqual (array (1,1,1), 222);
     }
+
+    // permuteIndices tests
+    void testPermute ()
+    {   
+        array3.reshape(difference3_type(3,5,7));
+        for(int k=0; k<array3.size(); ++k)
+            array3[k] = k;
+
+        array3_type ref(difference3_type(array3.shape(2), array3.shape(0), array3.shape(1)));
+        for(int k=0; k<array3.shape(0); ++k)
+            for(int l=0; l<array3.shape(1); ++l)
+                for(int m=0; m<array3.shape(2); ++m)
+                    ref(m,k,l) = array3(k,l,m);
+
+        MultiArrayView <3, scalar_type, StridedArrayTag>
+                array = array3.permuteDimensions (difference3_type (2, 0, 1));        
+        shouldEqual(ref.shape(), array.shape());
+        should(ref == array);
+
+        array2_type ref2(difference2_type(array3.shape(1), array3.shape(0)));
+        for(int k=0; k<array3.shape(0); ++k)
+            for(int l=0; l<array3.shape(1); ++l)
+                ref2(l, k) = array3(k, l, 0);
+
+        MultiArrayView <2, scalar_type, StridedArrayTag>
+                array2 = array3.bindOuter(0).transpose ();
+        shouldEqual(ref2.shape(), array2.shape());
+        should(ref2 == array2);
+
+        try {
+            array3.permuteDimensions (difference3_type (2, 0, 0));   
+            failTest("no exception thrown");
+        }
+        catch(vigra::ContractViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nMultiArrayView::permuteDimensions(): every dimension must occur exactly once");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+    }
+
+    void testNorm ()
+    {
+        shouldEqual(array3.squaredNorm(), 332833500);
+        
+        shouldEqual(array3.norm(), std::sqrt(332833500.0));
+        shouldEqual(array3.norm(0), 999.0);
+        shouldEqual(array3.norm(1), 499500.0);
+        shouldEqualTolerance(array3.norm(2, false), std::sqrt(332833500.0), 1e-14);
+        
+        difference3_type first(0,0,0), last(0,1,1);
+        shouldEqual(array3.subarray(first, last).norm(), 0.0);
+        shouldEqual(array3.subarray(first, last).norm(0), 0.0);
+        shouldEqual(array3.subarray(first, last).norm(1), 0.0);
+        shouldEqual(array3.subarray(first, last).norm(2, false), 0.0);
+
+        shouldEqual(array3.squaredNorm(), squaredNorm(array3));
+        shouldEqual(array3.norm(), norm(array3));
+    }
+    
+    void testScanOrderAccess()
+    {
+        shouldEqual(array3.size(), 1000);
+        for(int k=0; k< array3.size(); ++k)
+        {
+            shouldEqual(array3[k], k);
+            shouldEqual(array3[array3.scanOrderIndexToCoordinate(k)], k);
+            shouldEqual(array3.coordinateToScanOrderIndex(array3.scanOrderIndexToCoordinate(k)), k);
+        }
+            
+        MultiArrayView <2, scalar_type, StridedArrayTag> array = array3.bindInner(2);
+        shouldEqual(array.size(), 100);
+        for(int k=0; k< array.size(); ++k)
+            shouldEqual(array[k], 10*k+2);
+            
+        typedef MultiArrayView <2, scalar_type, UnstridedArrayTag>::difference_type Shape;
+        MultiArrayView <2, scalar_type, UnstridedArrayTag>
+            subarray = array3.bindOuter(2).subarray(Shape(1,0), Shape(10,9));
+        shouldEqual(subarray.size(), 81);
+        for(int k=0, l=200; k< subarray.size(); ++k, ++l)
+        {
+            if(k%9 == 0)
+                ++l;
+            shouldEqual(subarray[k], l);
+        }
+    }
+    
+    void testAssignmentAndReset()
+    {
+        typedef MultiArrayView <3, scalar_type, UnstridedArrayTag>::difference_type Shape;
+        MultiArrayView <3, scalar_type, UnstridedArrayTag> array;
+        array = array3;
+        should(array3 == array);
+        try {
+            array = array3.subarray(Shape(0,0,0), Shape(10,1,1));
+            failTest("no exception thrown");
+        }
+        catch(vigra::ContractViolation & c)
+        {
+            std::string expected("\nPrecondition violation!\nMultiArrayView::operator=(MultiArrayView const &) size mismatch");
+            std::string message(c.what());
+            should(0 == expected.compare(message.substr(0,expected.size())));
+        }
+        MultiArrayView <3, scalar_type, UnstridedArrayTag> subarray = array3.subarray(Shape(0,0,0), Shape(10,1,1));
+        subarray = array3.subarray(Shape(0,1,0), Shape(10,2,1)); // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0));
+        subarray += array3.subarray(Shape(0,1,0), Shape(10,2,1)); // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), 2.0*array3(k,1,0));
+        subarray -= array3.subarray(Shape(0,1,0), Shape(10,2,1)); // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0));
+        subarray *= array3.subarray(Shape(0,1,0), Shape(10,2,1)); // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), sq(array3(k,1,0)));
+        subarray /= array3.subarray(Shape(0,1,0), Shape(10,2,1)); // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0));
+            
+        subarray += 1; // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0) + 1);
+        subarray -= 2; // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0) - 1);
+        subarray *= 2; // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), 2*(array3(k,1,0) - 1));
+        subarray /= 2; // should overwrite the data
+        for(unsigned int k=0; k<10; ++k)
+            shouldEqual(array3(k,0,0), array3(k,1,0) - 1);
+    }
 };
 
 
@@ -201,6 +355,13 @@ public:
     {
         array1_t a (shape1_t(2), 1);
         shouldEqual (a.shape (0), 2);
+        shouldEqual (a(0), 1);
+        shouldEqual (a(1), 1);
+        
+        a.init(2);
+        shouldEqual(a(0), 2);
+        shouldEqual(a(1), 2);
+        should(a == array1_t(shape1_t(2), 4).init(2));
     }
 
     void test_assignment ()
@@ -399,11 +560,16 @@ public:
 
     void test_reshape ()
     {
+        shouldEqual(a3(0,0), 1);
+        a3.reshape(a3.shape());
+        shouldEqual(a3(0,0), 0);
+    
         a3.reshape (shape3_t(20,30,50));
 
         shouldEqual (a3.shape (0), 20);
         shouldEqual (a3.shape (1), 30);
         shouldEqual (a3.shape (2), 50);
+        shouldEqual(a3(0,0), 0);
     }
 
     void test_subarray ()
@@ -582,7 +748,7 @@ struct MultiImpexTest
         shouldEqual(result(0,1,2), 3);
         shouldEqual(result(0,1,3), 4);
 
-		exportVolume(array, std::string("impex/test"), std::string(".png"));
+        exportVolume(array, std::string("impex/test"), std::string(".png"));
         
         importVolume(result, std::string("impex/test"), std::string(".png"));
         
@@ -789,7 +955,7 @@ struct ImageTest
         img.init(0);
         for(; i != img.end(); ++i)
         {
-			should(acc(i) == Value(0));
+            should(acc(i) == Value(0));
         }
         img(1,1) = Value(200);
         img1 = img;
@@ -900,6 +1066,7 @@ struct MultiArrayDataTestSuite
     MultiArrayDataTestSuite()
     : vigra::test_suite("MultiArrayDataTestSuite")
     {
+        add( testCase( &MultiArrayDataTest::testEquality ) );
         add( testCase( &MultiArrayDataTest::test_subarray ) );
         add( testCase( &MultiArrayDataTest::test_stridearray ) );
         add( testCase( &MultiArrayDataTest::test_bindOuter ) );
@@ -907,6 +1074,10 @@ struct MultiArrayDataTestSuite
         add( testCase( &MultiArrayDataTest::test_bindAt ) );
         add( testCase( &MultiArrayDataTest::test_bind ) );
         add( testCase( &MultiArrayDataTest::test_bind0 ) );
+        add( testCase( &MultiArrayDataTest::testPermute ) );
+        add( testCase( &MultiArrayDataTest::testNorm ) );
+        add( testCase( &MultiArrayDataTest::testScanOrderAccess ) );
+        add( testCase( &MultiArrayDataTest::testAssignmentAndReset ) );
         add( testCase( &MultiArrayNavigatorTest::testNavigator ) );
     }
 };
