@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -36,7 +35,7 @@
 /************************************************************************/
 
 // -*- c++ -*-
-// $Id: test.cxx,v 1.9 2005/12/15 23:11:47 koethe Exp $
+// $Id$
 
 #include "unittest.hxx"
 #include "vigra/multi_array.hxx"
@@ -63,7 +62,7 @@ struct MultiArrayPointoperatorsTest
     MultiArrayPointoperatorsTest()
     : img(Size3(5,4,3))
     {
-        unsigned int i;
+        int i;
         PixelType c = 0.1;
         for(i=0; i<img.elementCount(); ++i, ++c)
             img.data()[i] = c;
@@ -367,6 +366,98 @@ struct MultiArrayPointoperatorsTest
                 for(x=0; x<img.shape(0); ++x)
                     shouldEqual(res(x,y,z), 3.0*img(x,y,z));
     }
+    
+    void testInitMultiArrayBorder(){
+        typedef vigra::MultiArray<1,int> IntLine;
+        typedef vigra::MultiArray<2,int> IntImage;
+        typedef vigra::MultiArray<3,int> IntVolume;
+        
+        const int desired_vol[] ={  0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 5, 5, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0,
+                                    0, 0, 0, 0, 0, 0};
+
+        const int desired_img[] ={  0, 0, 0, 0, 0, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 5, 5, 5, 5, 0,
+                                    0, 0, 0, 0, 0, 0};
+
+        const int desired_lin[] ={  0, 0, 0, 5, 0, 0, 0 };
+
+        const int desired_vol2[] ={  0, 0,
+                                     0, 0,
+
+                                     0, 0, 
+                                     0, 0};
+
+        IntVolume vol(IntVolume::difference_type(6,6,6));
+        
+        for(IntVolume::iterator iter=vol.begin(); iter!=vol.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(vol),2,0);
+        shouldEqualSequence(vol.begin(), vol.end(), desired_vol);
+
+        IntImage img(IntImage::difference_type(6,6));
+        
+        for(IntImage::iterator iter=img.begin(); iter!=img.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(img),1,0);
+        shouldEqualSequence(img.begin(), img.end(), desired_img);
+
+        IntLine lin(IntLine::difference_type(7));
+        
+        for(IntLine::iterator iter=lin.begin(); iter!=lin.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(lin),3,0);
+        shouldEqualSequence(lin.begin(), lin.end(), desired_lin);
+
+        IntVolume vol2(IntVolume::difference_type(2,2,2));
+        
+        for(IntVolume::iterator iter=vol2.begin(); iter!=vol2.end(); ++iter)
+            *iter=5;
+        initMultiArrayBorder(destMultiArrayRange(vol2),9,0);
+        shouldEqualSequence(vol2.begin(), vol2.end(), desired_vol2);
+
+    }
 };
 
 struct MultiArraySeparableConvolutionTest
@@ -388,10 +479,10 @@ struct MultiArraySeparableConvolutionTest
 
   void makeWedge( Image3D &image )
   {
-    const Size3 size = image.shape();
-    const int width = size[0];
-    const int height = size[1];
-    const int depth = size[2];
+    const Size3 shape = image.shape();
+    const int width = shape[0];
+    const int height = shape[1];
+    const int depth = shape[2];
     for( int z = 0; z < depth; ++z ) 
     {
       for( int y = 0; y < height; ++y ) 
@@ -409,10 +500,10 @@ struct MultiArraySeparableConvolutionTest
   void makeBox( Image3D &image )
   {
     const int b = 8;
-    const Size3 size = image.shape();
-    const int width = size[0];
-    const int height = size[1];
-    const int depth = size[2];
+    const Size3 shape = image.shape();
+    const int width = shape[0];
+    const int height = shape[1];
+    const int depth = shape[2];
     for( int z = 0; z < depth; ++z ) 
     {
       for( int y = 0; y < height; ++y ) 
@@ -438,11 +529,11 @@ struct MultiArraySeparableConvolutionTest
 
   void test_1DValidity( const Image3D &src, float ksize )
   {
-    Image3D d1( src.size() );
-    Image3D dn( src.size() );
-    Image3D dest3( src.size() );
+    Image3D d1( src.shape() );
+    Image3D dn( src.shape() );
+    Image3D dest3( src.shape() );
 
-    int depth = src.size()[2];
+    int depth = src.shape()[2];
 
     for( int d = 0; d < 3; ++d ) 
     {
@@ -467,10 +558,10 @@ struct MultiArraySeparableConvolutionTest
 
   void test_1DValidityB( const Image3D &src, float ksize )
   {
-    Image3D dst1( src.size() );
-    Image3D dst2( src.size() );
+    Image3D dst1( src.shape() );
+    Image3D dst2( src.shape() );
 
-    const int depth = src.size()[2];
+    const int depth = src.shape()[2];
 
     vigra::Kernel1D<float> kernx;
     vigra::Kernel1D<float> kerny;
@@ -521,10 +612,10 @@ struct MultiArraySeparableConvolutionTest
 
   void test_2DValidity( const Image3D &src, float ksize )
   {
-    Image3D d2( src.size() );
-    Image3D dn( src.size() );
+    Image3D d2( src.shape() );
+    Image3D dn( src.shape() );
 
-    int depth = src.size()[2];
+    int depth = src.shape()[2];
 
     std::vector<vigra::Kernel1D<float> > kernels( 3 );
     kernels[0].initGaussian( ksize );
@@ -554,8 +645,8 @@ struct MultiArraySeparableConvolutionTest
 
   void test_inplacenessN( const Image3D &src, float ksize )
   {
-    Image3D da( src.size() );
-    Image3D db( src.size() );
+    Image3D da( src.shape() );
+    Image3D db( src.shape() );
 
     std::vector<vigra::Kernel1D<float> > kernels( 3 );
     kernels[0].initGaussian( ksize );
@@ -581,8 +672,8 @@ struct MultiArraySeparableConvolutionTest
 
   void test_inplaceness1( const Image3D &src, float ksize, bool useDerivative )
   {
-    Image3D da( src.size() );
-    Image3D db( src.size() );
+    Image3D da( src.shape() );
+    Image3D db( src.shape() );
 
     Kernel1D<float> kernel;
     if( ! useDerivative )
@@ -618,8 +709,8 @@ struct MultiArraySeparableConvolutionTest
   {
     const double sigma = kernelSize/2;
     const int b = useGaussian ? int( 0.5 + 3*sigma ) : 1;
-    Image3D src( base.size() );
-    Image3x3 grad( src.size() );
+    Image3D src( base.shape() );
+    Image3x3 grad( src.shape() );
     makeWedge( src );
 
     if( ! useGaussian )
@@ -633,10 +724,10 @@ struct MultiArraySeparableConvolutionTest
     v[0] = 1; v[1] = 1; v[2] = 1;
     const float v2 = dot(v,v);
 
-    const Size3 size = src.shape();
-    const int width = size[0];
-    const int height = size[1];
-    const int depth = size[2];
+    const Size3 shape = src.shape();
+    const int width = shape[0];
+    const int height = shape[1];
+    const int depth = shape[2];
     for( int z = b; z < depth-b; ++z ) 
     {
       for( int y = b; y < height-b; ++y ) 
@@ -654,8 +745,8 @@ struct MultiArraySeparableConvolutionTest
   void test_gradient_magnitude( const Image3D &src )
   {
     // just a test for mere compileability
-    Image3D dst( src.size() );
-    Image3x3 grad( src.size() );
+    Image3D dst( src.shape() );
+    Image3x3 grad( src.shape() );
     symmetricGradientMultiArray( srcMultiArrayRange(src),
                                  destMultiArray(grad) );
 
@@ -666,13 +757,13 @@ struct MultiArraySeparableConvolutionTest
 
   //--------------------------------------------
 
-  const Size3 size;
+  const Size3 shape;
   Image3D srcImage;
   const float kernelSize;
 
   MultiArraySeparableConvolutionTest()
-    : size( 60, 70, 50 ),
-      srcImage( size ),
+    : shape( 60, 70, 50 ),
+      srcImage( shape ),
       kernelSize( 1.8 )
   {
     makeBox( srcImage );
@@ -735,6 +826,7 @@ struct MultiArrayPointOperatorsTestSuite
         add( testCase( &MultiArrayPointoperatorsTest::testCombine2OuterReduce ) );
         add( testCase( &MultiArrayPointoperatorsTest::testCombine2InnerReduce ) );
         add( testCase( &MultiArrayPointoperatorsTest::testCombine3 ) );
+        add( testCase( &MultiArrayPointoperatorsTest::testInitMultiArrayBorder ) );
     }
 }; // struct MultiArrayPointOperatorsTestSuite
 
