@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                     */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -372,15 +371,15 @@ struct PixelTypesTestSuite
 {
     typedef TinyVectorTest<vigra::TinyVector<unsigned char, 3>,
                            vigra::TinyVector<int, 3>,
-                           vigra::TinyVector<float, 3> > TinyVectorTest;
+                           vigra::TinyVector<float, 3> > TinyVectorTests;
 
     PixelTypesTestSuite()
     : vigra::test_suite("PixelTypesTest")
     {
-        add( testCase(&TinyVectorTest::testConstruction));
-        add( testCase(&TinyVectorTest::testComparison));
-        add( testCase(&TinyVectorTest::testArithmetic));
-        add( testCase(&TinyVectorTest::testAccessor));
+        add( testCase(&TinyVectorTests::testConstruction));
+        add( testCase(&TinyVectorTests::testComparison));
+        add( testCase(&TinyVectorTests::testArithmetic));
+        add( testCase(&TinyVectorTests::testAccessor));
 
         add( testCase(&RGBValueTest::testConstruction));
         add( testCase(&RGBValueTest::testComparison));
