@@ -4,12 +4,11 @@
 /*       Cognitive Systems Group, University of Hamburg, Germany        */
 /*                                                                      */
 /*    This file is part of the VIGRA computer vision library.           */
-/*    ( Version 1.5.0, Dec 07 2006 )                                    */
 /*    The VIGRA Website is                                              */
 /*        http://kogs-www.informatik.uni-hamburg.de/~koethe/vigra/      */
 /*    Please direct questions, bug reports, and contributions to        */
-/*        koethe@informatik.uni-hamburg.de          or                  */
-/*        vigra@kogs1.informatik.uni-hamburg.de                         */
+/*        ullrich.koethe@iwr.uni-heidelberg.de    or                    */
+/*        vigra@informatik.uni-hamburg.de                               */
 /*                                                                      */
 /*    Permission is hereby granted, free of charge, to any person       */
 /*    obtaining a copy of this software and associated documentation    */
@@ -51,6 +50,8 @@
 #include "vigra/watersheds.hxx"
 #include "vigra/noise_normalization.hxx"
 #include "vigra/slanted_edge_mtf.hxx"
+#include "vigra/affinegeometry.hxx"
+#include "vigra/affine_registration.hxx"
 #include "vigra/impex.hxx"
 
 using namespace vigra;
@@ -1381,7 +1382,7 @@ struct InterestOperatorTest
                    0.0, 0.0, 0.0,  0.0, 0.0,  0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,  0.0, 0.0,  0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,  0.0, 1.0,  0.0, 0.0, 0.0, 0.0,
-                   0.0, 0.0, 1.0, -1.0, 0.0, -1.0, 1.0, 0.0, 0.0,
+                   0.0, 0.0, 0.0, -1.0, 0.0, -1.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,  0.0, 1.0,  0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,  0.0, 0.0,  0.0, 0.0, 0.0, 0.0,
                    0.0, 0.0, 0.0,  0.0, 0.0,  0.0, 0.0, 0.0, 0.0,
@@ -1548,22 +1549,22 @@ struct SlantedEdgeMTFTest
         importImage(info, destImage(image));
         
         reference.push_back(Pair(0, 1));
-        reference.push_back(Pair(0.0564351, 0.961336));
-        reference.push_back(Pair(0.11287, 0.905812));
-        reference.push_back(Pair(0.169305, 0.831278));
-        reference.push_back(Pair(0.225741, 0.740566));
-        reference.push_back(Pair(0.282176, 0.641225));
-        reference.push_back(Pair(0.338611, 0.541988));
-        reference.push_back(Pair(0.395046, 0.448508));
-        reference.push_back(Pair(0.451481, 0.363593));
-        reference.push_back(Pair(0.507916, 0.287531));
-        reference.push_back(Pair(0.564351, 0.220531));
-        reference.push_back(Pair(0.620787, 0.163246));
-        reference.push_back(Pair(0.677222, 0.115963));
-        reference.push_back(Pair(0.733657, 0.0777855));
-        reference.push_back(Pair(0.790092, 0.0468734));
-        reference.push_back(Pair(0.846527, 0.0211264));
-        reference.push_back(Pair(0.902962, 0));
+        reference.push_back(Pair(0.0564351, 0.981739));
+        reference.push_back(Pair(0.11287, 0.929577));
+        reference.push_back(Pair(0.169305, 0.850509));
+        reference.push_back(Pair(0.22574, 0.754266));
+        reference.push_back(Pair(0.282175, 0.651081));
+        reference.push_back(Pair(0.33861, 0.549492));
+        reference.push_back(Pair(0.395045, 0.454718));
+        reference.push_back(Pair(0.45148, 0.368628));
+        reference.push_back(Pair(0.507915, 0.291512));
+        reference.push_back(Pair(0.564351, 0.223585));
+        reference.push_back(Pair(0.620786, 0.165506));
+        reference.push_back(Pair(0.677221, 0.117569));
+        reference.push_back(Pair(0.733656, 0.0788625));
+        reference.push_back(Pair(0.790091, 0.0475224));
+        reference.push_back(Pair(0.846526, 0.021419));
+        reference.push_back(Pair(0.902961, 0));
     }
     
     void testSlantedEdgeMTF()
@@ -1583,6 +1584,101 @@ struct SlantedEdgeMTFTest
     }
 };
 
+struct AffineRegistrationTest
+{
+    typedef vigra::DImage Image;
+    typedef vigra::TinyVector<double, 2> Vector2;
+    typedef vigra::ArrayVector<Vector2> PointList;
+    
+    Image image;
+
+    AffineRegistrationTest()
+    {
+        ImageImportInfo info("lenna128.xv");
+        image.resize(info.size());
+        importImage(info, destImage(image));
+    }
+    
+    void testCorrespondingPoints()
+    {
+        Matrix<double> point(3,1), res(3,1);
+        point(2,0) = 1.0;
+        
+        PointList src(3), dest(3);
+        src[0] = Vector2(1.6, 2.9);     
+        src[1] = Vector2(-1.3, -4.0);     
+        src[2] = Vector2(7.1, -0.4);     
+        dest[0] = Vector2(-2.9, 1.6);     
+        dest[1] = Vector2(12.6, -3.4);     
+        dest[2] = Vector2(-3.3, 4.2);
+        
+        for(int k=1; k<=3; ++k)
+        {     
+            Matrix<double> a = affineMatrix2DFromCorrespondingPoints(src.begin(), src.begin()+k, dest.begin());
+            for(int i=0; i<k; ++i)
+            {
+                point(0,0) = src[i][0];
+                point(1,0) = src[i][1];
+                res = a * point;
+                shouldEqualTolerance(res(0,0), dest[i][0], 1e-14);
+                shouldEqualTolerance(res(1,0), dest[i][1], 1e-14);
+                shouldEqual(res(2,0), 1.0);
+            }
+        }
+    }
+
+    void testTranslationRegistration()
+    {
+        Matrix<double> m = translationMatrix2D(Vector2(5.0, 10.0));
+        Image timg(image.size());
+        affineWarpImage(SplineImageView<2, double>(srcImageRange(image)), destImageRange(timg), m);
+        
+        Matrix<double> estimated = identityMatrix<double>(3);
+        estimateTranslation(srcImageRange(image), srcImageRange(timg), estimated,
+                            AffineMotionEstimationOptions<1>().highestPyramidLevel(3));
+        
+        for(int i=0; i<9; ++i)
+            shouldEqualTolerance(m.data()[i] - estimated.data()[i], 0.0, 1e-6);
+    }
+
+    void testSimilarityRegistration()
+    {
+        Matrix<double> m = translationMatrix2D(Vector2(5.0, 10.0)) * 
+                             rotationMatrix2DDegrees(5.0)* scalingMatrix2D(0.9);
+        Image timg(image.size());
+        affineWarpImage(SplineImageView<2, double>(srcImageRange(image)), destImageRange(timg), m);
+
+        Matrix<double> estimated = identityMatrix<double>(3);
+        estimateSimilarityTransform(srcImageRange(image), srcImageRange(timg), estimated,
+                            AffineMotionEstimationOptions<>().useLaplacianPyramid(false));
+        
+        for(int i=0; i<9; ++i)
+            shouldEqualTolerance(m.data()[i] - estimated.data()[i], 0.0, 1e-6);
+
+        estimated = identityMatrix<double>(3);
+        estimateSimilarityTransform(srcImageRange(image), 
+                            srcIterRange(timg.upperLeft(), timg.lowerRight()-Diff2D(20,20)), estimated,
+                            AffineMotionEstimationOptions<>().useLaplacianPyramid(true));
+        
+        for(int i=0; i<9; ++i)
+            shouldEqualTolerance(m.data()[i] , estimated.data()[i], 1e-2);
+    }
+
+    void testAffineRegistration()
+    {
+        Matrix<double> m = translationMatrix2D(Vector2(5.0, 10.0)) * 
+                             rotationMatrix2DDegrees(5.0)* scalingMatrix2D(1.0, 0.9);
+        Image timg(image.size());
+        affineWarpImage(SplineImageView<2, double>(srcImageRange(image)), destImageRange(timg), m);
+
+        Matrix<double> estimated = identityMatrix<double>(3);
+        estimateAffineTransform(srcImageRange(image), srcImageRange(timg), estimated);
+        
+        for(int i=0; i<9; ++i)
+            shouldEqualTolerance(m.data()[i] - estimated.data()[i], 0.0, 1e-6);
+    }
+};
+
 struct SimpleAnalysisTestSuite
 : public vigra::test_suite
 {
@@ -1636,6 +1732,10 @@ struct SimpleAnalysisTestSuite
         add( testCase( &NoiseNormalizationTest::testParametricNoiseNormalizationRGB));
         add( testCase( &NoiseNormalizationTest::testNonparametricNoiseNormalizationRGB));
         add( testCase( &SlantedEdgeMTFTest::testSlantedEdgeMTF));
+        add( testCase( &AffineRegistrationTest::testCorrespondingPoints));
+        add( testCase( &AffineRegistrationTest::testTranslationRegistration));
+        add( testCase( &AffineRegistrationTest::testSimilarityRegistration));
+        add( testCase( &AffineRegistrationTest::testAffineRegistration));
     }
 };
 
